#include <iostream>
#include <cstdint>
#include <vector>
#include <algorithm>
//#include <cassert>

const uint32_t MAX_N = 102;

struct Frame {
	uint32_t x1, y1;
	uint32_t x2, y2;

	Frame(uint32_t px1 = 0, uint32_t py1 = 0, uint32_t px2 = 0, uint32_t py2 = 0):
		x1(px1), y1(py1), x2(px2), y2(py2) {}
};

std::vector<Frame> ans;

char table[MAX_N][MAX_N];
uint32_t w, h;

void input() {
	std::ios::sync_with_stdio(false);
	std::cin.tie(nullptr);
	std::cin >> h >> w;

	for (uint32_t y = 0;y < h;y++) {
		for (uint32_t x = 0;x < w;x++) {
			std::cin >> table[x][y];
		}
	}
}

void print_table() {
	for (uint32_t x = 0;x < w;x++) {
		std::cerr << "=";
	}
	std::cerr << "\n";
	for (uint32_t y = 0;y < h;y++) {
		for (uint32_t x = 0;x < w;x++) {
			std::cerr << table[x][y];
		}
		std::cerr << "\n";
	}
}

bool eq(char a, char b) {
	return (a == b) || (a == '?');
}



uint32_t hSum[MAX_N][MAX_N + 1];
uint32_t vSum[MAX_N][MAX_N + 1];

uint32_t hAct[MAX_N][MAX_N + 1];
uint32_t vAct[MAX_N][MAX_N + 1];


void recalc_h(uint32_t y) {
	hSum[y][0] = 0;
	hAct[y][0] = 0;
	for (uint32_t x = 0;x < w;x++) {
		hSum[y][x + 1] = hSum[y][x] + eq(table[x][y], '-');
		hAct[y][x + 1] = hAct[y][x] + (table[x][y] == '-');
	}
}

void recalc_v(uint32_t x) {
	vSum[x][0] = 0;
	vAct[x][0] = 0;
	for (uint32_t y = 0;y < h;y++) {
		vSum[x][y + 1] = vSum[x][y] + eq(table[x][y], '|');
		vAct[x][y + 1] = vAct[x][y] + (table[x][y] == '|');
	}
}

void prep() {
	for (uint32_t x = 0;x < w;x++) {
		recalc_v(x);
	}
	for (uint32_t y = 0;y < h;y++) {
		recalc_h(y);
	}
}

bool h_ok(uint32_t y, uint32_t x1, uint32_t x2) {
	return (hSum[y][x2 + 1] - hSum[y][x1]) == (x2 - x1 + 1);
}

uint32_t h_act(uint32_t y, uint32_t x1, uint32_t x2) {
	return hAct[y][x2 + 1] - hAct[y][x1];
}

bool v_ok(uint32_t x, uint32_t y1, uint32_t y2) {
	return (vSum[x][y2 + 1] - vSum[x][y1]) == (y2 - y1 + 1);
}
uint32_t v_act(uint32_t x, uint32_t y1, uint32_t y2) {
	return vAct[x][y2 + 1] - vAct[x][y1];
}

void use_frame(uint32_t x1, uint32_t y1, uint32_t x2, uint32_t y2) {
//	assert(eq(table[x1][y1], '1')); 
//	assert(eq(table[x2][y1], '2')); 
//	assert(eq(table[x2][y2], '3')); 
//	assert(eq(table[x1][y2], '4')); 
//
//	for (uint32_t x = x1 + 1;x < x2;x++) {
//		assert(eq(table[x][y1], '-'));
//		assert(eq(table[x][y2], '-'));
//	}
//
//	for (uint32_t y = y1 + 1;y < y2;y++) {
//		assert(eq(table[x1][y], '|'));
//		assert(eq(table[x2][y], '|'));
//	}

	for (uint32_t x = x1;x <= x2;x++) {
		table[x][y1] = table[x][y2] = '?';
	}
	for (uint32_t y = y1;y <= y2;y++) {
		table[x1][y] = table[x2][y] = '?';
	}
	ans.push_back(Frame(x1, y1, x2, y2));

	prep();
	/*
	recalc_v(x1);
	r	prep();	prep();	prep();	prep();	prep();	prep();	prep();	prep();	prep();	prep();ecalc_v(x2);
	recalc_h(y1);
	r	prep();	prep();ecalc_h(y2);*/
	//std::cerr << "(" << x1 << " " << y1 << ") (" << x2 << " " << y2 <<  ")\n";
	//print_table();
}

inline bool ok_frame(const Frame &f) {
	if (!eq(table[f.x1][f.y1], '1')) {
		return false;
	}
	if (!eq(table[f.x2][f.y1], '2')) {
		return false;
	}
	if (!eq(table[f.x2][f.y2], '3')) {
		return false;
	}
	if (!eq(table[f.x1][f.y2], '4')) {
		return false;
	}
	if (!v_ok(f.x1, f.y1 + 1, f.y2 - 1)) {
		return false;
	} 
	if (!v_ok(f.x2, f.y1 + 1, f.y2 - 1)) {
		return false;
	}

	if (!h_ok(f.y1, f.x1 + 1, f.x2 - 1)) {
		return false;
	}
	if (!h_ok(f.y2, f.x1 + 1, f.x2 - 1)) {
		return false;
	} else {
		return true;
	}
}

inline uint32_t act_cnt(const Frame &f) {
	uint32_t curAct = 0;
	curAct += (table[f.x1][f.y1] == '1');
	curAct += (table[f.x2][f.y1] == '2');
	curAct += (table[f.x2][f.y2] == '3');
	curAct += (table[f.x1][f.y2] == '4');

	curAct += v_act(f.x1, f.y1 + 1, f.y2 - 1);
	curAct += v_act(f.x2, f.y1 + 1, f.y2 - 1);
	curAct += h_act(f.y1, f.x1 + 1, f.x2 - 1);
	curAct += h_act(f.y2, f.x1 + 1, f.x2 - 1);

	return curAct;
}

std::vector<Frame> possib_frames;

bool find_frames() {
	possib_frames.clear();
	for (uint32_t x1 = 0;x1 < w;x1++) {
		for (uint32_t y1 = 0;y1 < h;y1++) {
			if (!eq(table[x1][y1], '1')) {
				continue;
			}
			for (uint32_t x2 = x1 + 1;x2 < w;x2++) {
				if (table[x2][y1] == '-') {
					continue;
				}
				if (!eq(table[x2][y1], '2')) {
					break;
				} 
				for (uint32_t y2 = y1 + 1;y2 < h;y2++) {
					if (!eq(table[x1][y2], '4')) {
						if (table[x1][y2] != '|') {
							break;
						} else {
							continue;
						}
					}
					if (!eq(table[x2][y2], '3')) {
						if (table[x2][y2] != '|') {
							break;
						} else {
							continue;
						}
					}

					Frame f = Frame(x1, y1, x2, y2);
					if ((ok_frame(f)) && (act_cnt(f) > 0)) {
						possib_frames.push_back(f);
					}


					if (table[x1][y2] == '4') {
						break;
					}
					if (table[x2][y2] == '3') {
						break;
					}

				}
				if (table[x2][y1] == '2') {
					break;
				}
			}
		}
	}
	return possib_frames.size() > 0;
}

void empty_list() {
	while (true) {
		Frame best;
		uint32_t bestCnt = 0;
		std::vector<Frame> nList;
		for (Frame &f : possib_frames) {
			if (ok_frame(f)) {
				uint32_t c = act_cnt(f);
				if (c > 0) {
					nList.push_back(f);
				}
				//std::cerr << "act cnt " << c << std::endl;
				if (c > bestCnt) {
					bestCnt = c;
					best = f;
				}
			}
		}
		if (bestCnt == 0) {
			break;
		}
		use_frame(best.x1, best.y1, best.x2, best.y2);
		possib_frames = nList;
	}
}

void print_ans() {
	std::reverse(ans.begin(), ans.end());
	std::cout << ans.size() << "\n";
	for (const Frame &f : ans) {
		std::cout << f.y1 << " " << f.x1 << " " << f.y2 << " " << f.x2 << "\n";
	}
}

int main() {
	input();
	prep();
	while (find_frames()) {
		empty_list();
	}

	print_ans();
}
