#include <iostream>
#include <cstdint>
#include <vector> //shefa <3
#include <map> //Nakata <3
#include <utility>

//#define EVAL

struct Vertex {
	std::vector<Vertex*> inE;
	Vertex* outE;
	uint32_t country;
	uint32_t group;
	bool inLoop;
	bool vis;

	Vertex(): inLoop(false), vis(false) {}
};

std::vector<Vertex> graph;
void input() {
	std::ios::sync_with_stdio(false);
	std::cin.tie(nullptr);

	uint32_t cCnt;
	uint32_t vCnt;
	std::cin >> vCnt >> cCnt;

	graph.resize(vCnt);
	for (uint32_t i = 0;i < cCnt;i++) {
		uint32_t curCnt;
		std::cin >> curCnt;
		for (uint32_t j = 0;j < curCnt;j++) {
			uint32_t a;
			std::cin >> a;
			graph[a - 1].country = i;
		}
	}
	for (uint32_t i = 0;i < vCnt;i++) {
		uint32_t a;
		std::cin >> a;
		graph[i].outE = &graph[a - 1];
		graph[a - 1].inE.push_back(&graph[i]);
	}

}

std::vector<Vertex*> find_loop(Vertex *v) {
	std::vector<Vertex*> ans;
	v->vis = true;
	while (!v->outE->vis) {
		v = v->outE;
		v->vis = true;
	}
	for (Vertex *i = v->outE;i != v;i = i->outE) {
		i->inLoop = true;
		ans.push_back(i);
	}
	v->inLoop = true;
	ans.push_back(v);
	return ans;
}

//slowest part of the algorithm, to be optimized with hashing
uint32_t find_period(const std::vector<Vertex*> &loop) {
	for (uint32_t i = 1;i < loop.size();i++) {
		if (loop.size() % i) {
			continue;
		}
		bool ok = true;
		for (uint32_t j = i;j < loop.size();j++) {
			if (loop[j]->country != loop[j - i]->country) {
				ok = false;
				break;
			}
		}
		if (ok) {
			return i;
		}
	}
	return loop.size();
}

//typedef std::vector<std::pair<uint32_t, uint32_t>> CompSol; //country, targGroup

const uint32_t MAX_N = 2e5;

struct Hash {
	static const uint32_t BASE = 1000037;
	static const uint32_t MOD = 1e9 + 7;
	static uint64_t bPow[];
	static uint64_t iPow[];
	static uint64_t fast_pow(uint64_t a, uint64_t b, uint64_t m) {
		if (b == 0) {
			return 1;
		}
		uint64_t ans = fast_pow(a, b /2, m);
		ans = ans * ans % m;
		if (b & 1) {
			ans = ans * a % m;
		}
		return ans;
	}
	static void precomp();

	uint32_t val;
	uint32_t len;

	Hash(): val(0), len(0) {}
	Hash(uint32_t a): val(a), len(1) {}
	Hash operator+ (const Hash &a) const {
		Hash ans;
		ans.val = (val + a.val * bPow[len]) % MOD;
		ans.len = a.len + len;
		return ans;
	}
	Hash operator- (const Hash &a) const {
		Hash ans;
		ans.val = (val + MOD - a.val) % MOD;
		ans.val = ans.val * iPow[a.len] % MOD;
		ans.len = len - a.len;
		return ans;
	}
	bool operator== (const Hash &a) const {
		return ((a.len == len) && (a.val ==val));
	}
	bool operator< (const Hash &a) const {
		if (a.val != val) {
			return val < a.val;
		} else {
			return len < a.len;
		}
	}
};

uint64_t Hash::bPow[MAX_N];
uint64_t Hash::iPow[MAX_N];


void Hash::precomp() {
	bPow[0] = 1;
	for (uint32_t i = 1;i < MAX_N;i++) {
		bPow[i] = bPow[i - 1] * BASE % MOD;
	}
	iPow[0] = 1;
	for (uint32_t i = 1;i < MAX_N;i++) {
		iPow[i] = fast_pow(bPow[i], MOD - 2, MOD);
	}
}

typedef std::vector<std::pair<uint32_t, uint32_t>> LoopGroup; //country, group
//The Group from country .first going to group. second
std::map<std::pair<uint32_t, uint32_t>, uint32_t> countryGroup;

std::vector<LoopGroup> loopGroups;
std::map<Hash, LoopGroup*> lgMap;
uint32_t gCnt;
//country, to group
std::vector<std::pair<uint32_t, uint32_t>> ans;

void handle_tree(Vertex *r) {
	for (Vertex *v : r->inE) {
		if (v->inLoop) {
			continue;
		}
		v->vis = true;
		auto p = std::make_pair(v->country, r->group);
		if (countryGroup.find(p) == countryGroup.end()) {
			std::cerr << "CG (" << p.first << " " << p.second << ") -> " << gCnt << std::endl;
			countryGroup[p] = gCnt++;
			//ans.push_back(p);
		}
		v->group = countryGroup[p];
		handle_tree(v);
	}
}

uint32_t best_offset(const std::vector<Vertex*> &l, uint32_t p) {
	uint32_t best = 0;
	for (uint32_t i = 1;i < p;i++)  {
		bool better = false;
		for (uint32_t j = 0;j < p;j++) {
			uint32_t cPrev = l[(j + best) % p]->country;
			uint32_t cCur = l[(j + i) % p]->country;
			if (cCur != cPrev) {
				if (cCur < cPrev) {
					better = true;
				} else {
					better = false;
					break;
				}
			}
		}
		if (better) {
			best = i;
		}
	}
	return best;
}

bool lg_cmp(const LoopGroup &a, const LoopGroup &b) {
	if (a.size() != b.size()) {
		return false;
	}
	for (uint32_t i = 0;i < a.size();i++) {
		if (a[i].first != b[i].first) {
			return false;
		}
	}
	return true;
}

void set_loop_groups(const std::vector<Vertex*> &l, uint32_t p, uint32_t o) {
	LoopGroup cur;
	Hash curH;
	for (uint32_t i = 0;i < p;i++) {
		cur.push_back(std::make_pair(l[(i + o) % p]->country, 0));
		curH = curH + Hash(cur.back().first);
	}
	if (lgMap.find(curH) != lgMap.end()) {
		cur = *lgMap[curH];
	} else {
		for (uint32_t i = 0;i < cur.size();i++) {
			cur[i].second = gCnt + i;
			countryGroup[std::make_pair(cur[i].first, gCnt + (i + 1) % p)] = gCnt + i;
			//ans.push_back(std::make_pair(cur[i], 
		}
#ifndef EVAL
		for (auto &i : cur) {
			std::cerr << "(" << i.first << " " << i.second << ")" ;
		}
		std::cerr << std::endl;
#endif
		loopGroups.push_back(cur);
		lgMap[curH] = &loopGroups.back();

		gCnt += p;
	}
	for (uint32_t i = 0;i < l.size();i++) {
		l[(i + o) % l.size()]->group = cur[i % p].second;
	}
}

void handle_component(Vertex *start) {
	std::vector<Vertex*> loop = find_loop(start);
	
	uint32_t p = find_period(loop);
	uint32_t off = best_offset(loop, p);


#ifndef EVAL
	for (Vertex *v : loop) {
		std::cerr << v - &graph[0] + 1 << " ";
	}
	std::cerr << std::endl;
	std::cerr << "Period " << p << " offset " << off << std::endl;
#endif

	set_loop_groups(loop, p, off);
	//countryGroup.clear();
	//gCnt = p;
	//s = CompSol();

//	for (uint32_t i = 0;i < p;i++) {
//		auto pp = std::make_pair(l[i]->country, (i + 1) % p);
//		countryGroup[pp] = i;
//		ans.push_back(pp);
//	}
//	for (uint32_t i = 0;i < l.size();i++) {
//		l[i]->group = i % p;
//	}
	for (Vertex *v : loop) {
		handle_tree(v);
	}
}

void assign() {
	for (Vertex &v : graph) {
		if (!v.vis) {
			handle_component(&v);
		}
	}
}

void print_ans() {
	std::vector<std::vector<uint32_t>> ans;
	ans.resize(gCnt);
	for (uint32_t i = 0;i < graph.size();i++) {
		ans[graph[i].group].push_back(i + 1);

#ifndef EVAL
		std::cerr << ">> " << i + 1 << " " << graph[i].group << " " << graph[i].inLoop << std::endl;
#endif
	}
	std::cout << ans.size() << "\n";
	for (const auto &v : ans) {
		std::cout << v.size() << " ";
		for (const uint32_t &i : v) {
			std::cout << i << " ";
		}
		std::cout << "\n";
	}
}

int main() {
	Hash::precomp();
	loopGroups.reserve(MAX_N);
	input();
	assign();
	print_ans();
}
